<?php

/**
 * Класс для упрощения работы со средствами SAX встроеных в PHP.
 *
 * Создает дерево в виде массива, принимает путь к файлу
 * @author IvanSCM <ivanscm@fromru.com>
 */
class SAXRead {

    /**
     *
     * @var string $filename Имя файла для работы внутри класса
     */
    private $filename;

    /**
     *
     * @var resource $xml Дескриптор XML парсера
     */
    private $xml;

    /**
     *
     * @var array $tree Дерево документа, доступно после вызова ReadFile();
     */
    public $tree;

    /**
     *
     * @var array $stack Вспомогательный массив, стек
     */
    private $stack;

    /**
     * Конструктор
     *
     * Устанавливает имя файла для дальнейшей работы класса и регистрирует методы для обработчиков SAX
     * @param string $filename
     */
    function __construct($filename) {
        $this->stack = array();
        $this->filename = $filename;
        $this->xml = xml_parser_create('UTF-8');
        xml_set_element_handler($this->xml, 'self::StartTAG', 'self::EndTAG');
        xml_set_character_data_handler($this->xml, 'self::CharFind');
        //устанавливаем нижний регистр для тегов документа
        xml_parser_set_option($this->xml, XML_OPTION_CASE_FOLDING, false);
    }

    /**
     * Обработчик начального тега
     * @param resouce $xml
     * @param mixed $tag
     * @param mixed $attr
     */
    private function  StartTAG ($xml, $tag, $attr){
        // инициилизируем переменную узла и аттрибутов
        $node = array();
        $attributes = array();
        // устанавливаем имя тега
        $node["_name"] = $tag;
        //обход для добавления аттрибутов
        foreach ($attr as $key => $value) {
            $attributes[$key] = $value;
        }
        //добавляем аттрибуты
        $node['_attr'] = $attributes;

        $node["_value"]      = "";
        $node["_children"]  = array();
        //добавляем узел в стек
        array_push($this->stack, $node);
    }

    /**
     * Обработчик конечного тега
     * @param resource $xml
     * @param mixed $tag
     */
    private function EndTag($xml, $tag) {
        // в текущий узел ставим стек
        $node = array_pop($this->stack);
        // убераем пробелы в значении
        $node["_value"] = trim($node["_value"]);
        // добавляем текущий узел в стек
        $lastnode = count($this->stack);
        array_push($this->stack[$lastnode-1]["_children"], $node);
    }

    /**
     * Обработчик содержимого
     * @param resourse $xml
     * @param mixed $data
     */
    private function CharFind($xml, $data){
        // добавляем значение в текущий узел
        $lastnode = count($this->stack);
        $this->stack[$lastnode-1]["_value"] .= $data;
    }
    
    /**
     * Функция ReadFile
     * 
     * Создает структуру из файла
     * @return Array
     */
    function ReadFile(){
        //инициилизируем корень документа
        $this->StartTAG(null, 'root', array());
        // парсим
        xml_parse($this->xml, file_get_contents($this->filename));
        // результат заносим в переменную $tree класса для дальнейшего доступа
        $this->tree = array_pop($this->stack);
        return $this->tree;
    }

}
?>
